@@ -1,14 +1,14 @@
 /** @file
 	Parser: Array & Array_iterator classes decls.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_ARRAY_H
 #define PA_ARRAY_H
 
-#define IDENT_PA_ARRAY_H "$Id: pa_array.h,v 1.83 2015/10/26 01:21:55 moko Exp $"
+#define IDENT_PA_ARRAY_H "$Id: pa_array.h,v 1.85 2017/02/07 22:00:31 moko Exp $"
 
 // includes
 
@@ -169,6 +169,10 @@ public:
 
 	inline T operator [](size_t index) const { return get(index); }
 
+	inline void clear() {
+		fused=0;
+	}
+
 	/// iterate over all elements
 	template<typename I> void for_each(void (*callback)(T, I), I info) const {
 		T *last=felements+fused;
