@@ -1,14 +1,14 @@
 /** @file
 	Parser: commonly used functions.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_COMMON_H
 #define PA_COMMON_H
 
-#define IDENT_PA_COMMON_H "$Id: pa_common.h,v 1.161 2015/10/26 01:21:55 moko Exp $"
+#define IDENT_PA_COMMON_H "$Id: pa_common.h,v 1.174 2017/02/07 22:00:32 moko Exp $"
 
 #include "pa_string.h"
 #include "pa_hash.h"
@@ -16,29 +16,29 @@
 class Request;
 
 // defines
-#define HTTP_USER_AGENT	"user-agent"
-
-#define HTTP_STATUS	"status"
+#define HTTP_STATUS		"status"
 #define HTTP_STATUS_CAPITALIZED	"Status"
 
-#define HTTP_CONTENT_LENGTH	"content-length"
+#define HTTP_CONTENT_LENGTH		"content-length"
 #define HTTP_CONTENT_LENGTH_CAPITALIZED	"Content-Length"
 
 #define HTTP_CONTENT_TYPE		"content-type"
 #define HTTP_CONTENT_TYPE_UPPER		"CONTENT-TYPE"
 #define HTTP_CONTENT_TYPE_CAPITALIZED	"Content-Type"
-#define HTTP_CONTENT_TYPE_FORM_URLENCODED	"application/x-www-form-urlencoded"
-#define HTTP_CONTENT_TYPE_MULTIPART_FORMDATA	"multipart/form-data"
-#define HTTP_CONTENT_TYPE_MULTIPART_RELATED	"multipart/related"
-#define HTTP_CONTENT_TYPE_MULTIPART_MIXED	"multipart/mixed"
 
 #define CONTENT_DISPOSITION		"content-disposition"
 #define CONTENT_DISPOSITION_UPPER	"CONTENT-DISPOSITION"
 #define CONTENT_DISPOSITION_CAPITALIZED	"Content-Disposition"
-#define CONTENT_DISPOSITION_ATTACHMENT "attachment"
-#define CONTENT_DISPOSITION_INLINE "inline"
+
+#define CONTENT_DISPOSITION_ATTACHMENT	"attachment"
+#define CONTENT_DISPOSITION_INLINE	"inline"
 #define CONTENT_DISPOSITION_FILENAME_NAME "filename"
 
+#define HTTP_CONTENT_TYPE_FORM_URLENCODED	"application/x-www-form-urlencoded"
+#define HTTP_CONTENT_TYPE_MULTIPART_FORMDATA	"multipart/form-data"
+#define HTTP_CONTENT_TYPE_MULTIPART_RELATED	"multipart/related"
+#define HTTP_CONTENT_TYPE_MULTIPART_MIXED	"multipart/mixed"
+
 #define BASE64_STRICT_OPTION_NAME "strict"
 
 const String content_disposition_filename_name(CONTENT_DISPOSITION_FILENAME_NAME);
@@ -55,11 +55,11 @@ typedef HASH_STRING<Value*> HashStringValue;
 
 // replace system s*nprintf with our versions
 #undef vsnprintf 
-int __vsnprintf(char *, size_t, const char* , va_list);
-#define vsnprintf __vsnprintf 
+int pa_vsnprintf(char *, size_t, const char* , va_list);
+#define vsnprintf pa_vsnprintf 
 #undef snprintf
-int __snprintf(char *, size_t, const char* , ...);
-#define snprintf __snprintf
+int pa_snprintf(char *, size_t, const char* , ...);
+#define snprintf pa_snprintf
 
 #ifdef _MSC_VER
 
@@ -77,6 +77,21 @@ int __snprintf(char *, size_t, const char* , ...);
 #	define strncasecmp _strnicmp
 #endif
 
+#define stat __stat64
+#define pa_fstat _fstat64
+
+int pa_stat(const char *pathname, struct stat *buffer);
+int pa_open(const char *pathname, int flags, int mode=0);
+FILE *pa_fopen(const char *pathname, const char *mode);
+
+#else
+
+#define pa_stat stat
+#define pa_fstat fstat
+
+#define pa_open open
+#define pa_fopen fopen
+
 #endif
 
 /** 
@@ -94,11 +109,7 @@ void create_dir_for_file(const String& file_spec);
 
 int pa_get_valid_file_options_count(HashStringValue& options);
 
-typedef void (*File_read_action)(
-				struct stat& finfo,
-				int f, 
-				const String& file_spec, const char* fname, bool as_text,
-				void *context);
+typedef void (*File_read_action)(struct stat& finfo, int f, const String& file_spec, void *context);
 
 /**
 	shared-lock specified file, 
@@ -207,12 +218,8 @@ bool dir_exists(const String& file_spec);
 const String* file_exist(const String& path, const String& name);
 bool file_executable(const String& file_spec);
 
-bool file_stat(const String& file_spec, 
-				size_t& rsize, 
-				time_t& ratime,
-				time_t& rmtime,
-				time_t& rctime,
-				bool fail_on_read_problem=true);
+bool file_stat(const String& file_spec, uint64_t& rsize, time_t& ratime, time_t& rmtime, time_t& rctime, bool fail_on_read_problem=true);
+size_t check_file_size(uint64_t size, const String& file_spec);
 
 size_t stdout_write(const void *buf, size_t size);
 
@@ -224,8 +231,6 @@ int file_block_read(const int f, unsigned char* buffer, const size_t size);
 	String related functions
 */
 
-const char* capitalize(const char* s);
-
 /** under WIN32 "t" mode fixes DOS chars OK, 
 	can't say that about other systems/ line break styles
 */
@@ -235,6 +240,7 @@ char *getrow(char **row_ref,char delim='\n');
 char *lsplit(char *string, char delim);
 char *lsplit(char **string_ref,char delim);
 char *rsplit(char *string, char delim);
+
 const char* format(double value, const char *fmt);
 
 char* unescape_chars(const char* cp, int len, Charset* client_charset=0, bool js=false/*true==decode \uXXXX and don't convert '+' to space*/);
@@ -253,13 +259,12 @@ size_t strpos(const char *str, const char *substr);
 
 int remove_crlf(char *start, char *end);
 
-inline bool pa_isalpha(unsigned char c) {
-	return (((c>='A') && (c<='Z')) || ((c>='a') && (c<='z')));
-}
+inline bool pa_isalpha(unsigned char c) { return (((c>='A') && (c<='Z')) || ((c>='a') && (c<='z'))); }
+inline bool pa_isalnum(unsigned char c) { return (((c>='0') && (c<='9')) || pa_isalpha(c)); }
 
-inline bool pa_isalnum(unsigned char c) {
-	return (((c>='0') && (c<='9')) || pa_isalpha(c));
-}
+const char* capitalize(const char* s);
+char *str_lower(const char *s, size_t helper_length=0);
+char *str_upper(const char *s, size_t helper_length=0);
 
 const char* hex_string(unsigned char* bytes, size_t size, bool upcase);
 extern const char* hex_digits;
@@ -271,27 +276,14 @@ char* pa_base64_encode(const String& file_spec);
 const char *pa_idna_encode(const char *in, Charset &source);
 const char *pa_idna_decode(const char *in, Charset &source);
 
-const unsigned long pa_crc32(const char *in, size_t in_size);
-const unsigned long pa_crc32(const String& file_spec);
+unsigned long pa_crc32(const char *in, size_t in_size);
+unsigned long pa_crc32(const String& file_spec);
 
 /** 
 	Mix functions
 */
 
-// some stuff for use with .for_each
-static void copy_all_overwrite_to(
-								HashStringValue::key_type key, 
-								HashStringValue::value_type value, 
-								HashStringValue* dest) {
-	dest->put(key, value);
-}
-
-static void remove_key_from(
-							HashStringValue::key_type key, 
-							HashStringValue::value_type /*value*/, 
-							HashStringValue* dest) {
-	dest->remove(key);
-}
+#define PA_DEFAULT(A,B) ((A) ? (A):(B) )
 
 Charset* detect_charset(const char* content_type);
 
@@ -317,7 +309,7 @@ extern const String file_status_name;
 
 #ifndef DOXYGEN
 enum Table2hash_distint { D_ILLEGAL, D_FIRST };
-enum Table2hash_value_type { C_HASH, C_STRING, C_TABLE };
+enum Table2hash_value_type { C_HASH, C_STRING, C_TABLE, C_CODE };
 #endif
 
 #endif
