@@ -1,7 +1,7 @@
 /** @file
 	Parser: includes all Configure-d headers
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 
 	when used Configure [HAVE_CONFIG_H] it uses defines from Configure,
@@ -186,18 +186,24 @@ inline size_t min(size_t a, size_t b){ return a<b?a:b; }
 
 #endif
 
+#ifdef __GNUC__
+#  define PA_ATTR_UNUSED __attribute__((unused))
+#else
+#  define PA_ATTR_UNUSED
+#endif
+
 #ifdef WIN32
 #define THREAD_LOCAL __declspec(thread)
 #else
 #define THREAD_LOCAL // __thread // multithreading support required only for apache2 module
 #endif
 
-#if __clang__
+#ifdef __clang__
 #pragma clang diagnostic ignored "-Wparentheses"       // if(a=b)
-#pragma clang diagnostic ignored "-Winline-new-delete" // replacement operator cannot be declared 'inline'
+#pragma clang diagnostic ignored "-Wpointer-sign"      // CORD (unsigned char *) to char * in libcord
 #endif
 
-#if _MSC_VER
+#ifdef _MSC_VER
 #pragma warning(disable:4355)  // using this in calls to parent constructors
 #pragma warning(disable:4291)  // no matching operator delete found; memory will not be freed if initialization throws an exception
 #pragma warning(disable:4512)  // assignment operator could not be generated
