@@ -1,14 +1,14 @@
 /** @file
 	Parser: program executing for different OS-es decls.
 
-	Copyright (c) 2000-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2000-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_EXEC_H
 #define PA_EXEC_H
 
-#define IDENT_PA_EXEC_H "$Id: pa_exec.h,v 1.21 2015/10/26 01:21:55 moko Exp $"
+#define IDENT_PA_EXEC_H "$Id: pa_exec.h,v 1.23 2017/02/07 22:00:34 moko Exp $"
 
 #include "pa_string.h"
 #include "pa_hash.h"
@@ -29,6 +29,6 @@ PA_exec_result pa_exec(
 			const String& file_spec, 
 			const HashStringString* env, ///< 0 | Hash of String
 			const ArrayString& argv, ///< 0 | Array of command line arguments
-			String& in);
+			String::C in);
 
 #endif
