@@ -1,7 +1,7 @@
 /** @file
 	Parser: hash class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
@@ -17,7 +17,7 @@
 #ifndef PA_HASH_H
 #define PA_HASH_H
 
-#define IDENT_PA_HASH_H "$Id: pa_hash.h,v 1.94 2015/10/26 01:21:55 moko Exp $"
+#define IDENT_PA_HASH_H "$Id: pa_hash.h,v 1.98 2017/02/15 17:30:10 moko Exp $"
 
 #include "pa_memory.h"
 #include "pa_types.h"
@@ -210,7 +210,6 @@ public:
 				else
 					last=pair->prev;
 #endif
-				delete pair;
 				*ref=next;
 				--fpairs_count;
 				return true;
@@ -256,9 +255,12 @@ public:
 		return (first) ? first->value : V(0);
 	}
 
+	inline Pair* last_pair() const {
+		return (fpairs_count) ? (Pair*)((char *)last - offsetof(Pair, next)) : NULL;
+	}
+
 	String::Body last_key() const {
-		if (fpairs_count) {
-			Pair* pair = (Pair*)((char *)last - offsetof(Pair, next));
+		if(Pair* pair = last_pair()) {
 #ifdef HASH_CODE_CACHING
 			return String::Body(pair->key, pair->code);
 #else
@@ -270,7 +272,9 @@ public:
 	}
 
 	V last_value() const {
-		return (fpairs_count) ? ((Pair *)((char *)last - offsetof(Pair, next)))->value : V(0);
+		if(Pair* pair = last_pair())
+			return pair->value;
+		return NULL;
 	}
 
 	void order_clear() {
@@ -283,7 +287,6 @@ public:
 		*last=pair;
 		last=&(pair->next);
 	}
-
 #endif //HASH_ORDER
 
 	/// put a [value] under the [key] if that [key] existed @returns existed or not
@@ -505,7 +508,6 @@ public:
 				else
 					this->last=pair->prev;
 #endif
-				delete pair;
 				*ref=next;
 				--this->fpairs_count;
 				return true;
@@ -677,7 +679,7 @@ public:
 #endif
 
 		operator bool () {
-			return fcurrent != 0;
+			return fcurrent != NULL;
 		}
 
 		String::Body key(){
@@ -697,6 +699,42 @@ public:
 		}
 	};
 
+#ifdef HASH_ORDER
+	/// simple reverse hash iterator
+	class ReverseIterator {
+		const HASH_STRING<V>& fhash;
+		Pair *fcurrent;
+	public:
+		ReverseIterator(const HASH_STRING<V>& ahash): fhash(ahash) {
+			fcurrent=fhash.last_pair();
+		}
+
+		void prev() {
+			fcurrent=(fcurrent->prev == &fhash.first) ? NULL : (Pair*)((char *)fcurrent->prev - offsetof(Pair, next));
+		}
+
+		operator bool () {
+			return fcurrent != NULL;
+		}
+
+		String::Body key(){
+#ifdef HASH_CODE_CACHING
+			return String::Body(fcurrent->key, fcurrent->code);
+#else
+			return fcurrent->key;
+#endif
+		}
+
+		V value(){
+			return fcurrent->value;
+		}
+
+		Pair *pair(){
+			return fcurrent;
+		}
+	};
+#endif
+
 };
 
 #ifndef HASH_ORDER
