@@ -1,14 +1,14 @@
 /** @file
 	Parser: request class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_REQUEST_H
 #define PA_REQUEST_H
 
-#define IDENT_PA_REQUEST_H "$Id: pa_request.h,v 1.221 2015/10/26 01:21:56 moko Exp $"
+#define IDENT_PA_REQUEST_H "$Id: pa_request.h,v 1.249 2017/02/07 22:00:35 moko Exp $"
 
 #include "pa_pool.h"
 #include "pa_hash.h"
@@ -21,7 +21,6 @@
 
 // consts
 
-const uint ANTI_ENDLESS_EXECUTE_RECOURSION=1000;
 const size_t pseudo_file_no__process=1;
 
 // forwards
@@ -36,6 +35,10 @@ class VCookie;
 class VStateless_class;
 class VConsole;
 
+extern int pa_loop_limit;
+extern int pa_execute_recoursion_limit;
+extern size_t pa_file_size_limit;
+
 /// Main workhorse.
 class Request: public PA_Object {
 	friend class Temp_lang;
@@ -62,8 +65,10 @@ public:
 
 	enum Skip {
 		SKIP_NOTHING,
+		SKIP_CONTINUE,
 		SKIP_BREAK,
-		SKIP_CONTINUE
+		SKIP_RETURN,
+		SKIP_INTERRUPTED
 	};
 
 private:
@@ -86,7 +91,7 @@ private:
 
 		/// needed to fill unused Array entries
 		StackItem() {}
-		StackItem(Value& avalue): fvalue(&avalue) {}		
+		StackItem(Value& avalue): fvalue(&avalue) {}
 		StackItem(ArrayOperation& aops): fops(&aops) {}
 		StackItem(VMethodFrame& amethod_frame): fmethod_frame(&amethod_frame) {}
 	};
@@ -112,7 +117,7 @@ private:
 	///@{ core data
 
 	/// classes
-	HashString<Value*> fclasses;
+	HashString<VStateless_class*> fclasses;
 
 	/// already used files to avoid cyclic uses
 	HashString<bool> used_files;
@@ -120,10 +125,8 @@ private:
 	/// list of all used files, Operation::file_no = index to it
 	Array<String::Body> file_list;
 
-	/**	endless execute(execute(... preventing counter 
-		@see ANTI_ENDLESS_EXECUTE_RECOURSION
-	*/
-	uint anti_endless_execute_recoursion;
+	/// endless execute(execute(... preventing counter
+	int anti_endless_execute_recoursion;
 
 	///@}
 
@@ -142,14 +145,16 @@ public:
 	Value* rcontext;
 	WContext* wcontext;
 	/// current language
-	String::Language flang; 
+	String::Language flang;
 	/// current connection
 	SQL_Connection* fconnection;
 	//@}
-	/// interrupted flag, raised on signals [SIGPIPE]
-	bool finterrupted;
-	Skip fskip;
+
+private:
+
 	int fin_cycle;
+	Skip fskip;
+	VMethodFrame* freturn_method_frame;
 
 public:
 	uint register_file(String::Body file_spec);
@@ -180,111 +185,98 @@ public:
 	~Request();
 
 	/// global classes
-	HashString<Value*>& classes() { return fclasses; }
-	Value* get_class(const String& name);
+	HashString<VStateless_class*>& classes() { return fclasses; }
+	VStateless_class* get_class(const String& name);
+	void put_class(VStateless_class *aclass){ classes().put(aclass->type(), aclass); }
 
 	/**
 		core request processing
-
 		BEWARE: may throw exception to you: catch it!
 	*/
-	void core(
-		const char* config_filespec, ///< system config filespec
-		bool config_fail_on_read_problem, ///< fail if system config file not found
-		bool header_only);
+	void core(const char* config_filespec, bool config_fail_on_read_problem, bool header_only);
 
 	/// executes ops
 	void execute(ArrayOperation& ops); // execute.C
-	void op_call(VMethodFrame &frame);
-	void op_call_write(VMethodFrame &frame);
-	Value& construct(Value &class_value, const Method &method);
+
+	template<typename Frame> void call(Frame& frame){
+		VMethodFrame *saved_method_frame=method_frame;
+		Value* saved_rcontext=rcontext;
+		WContext *saved_wcontext=wcontext;
+
+		rcontext=wcontext=method_frame=&frame;
+
+		frame.call(*this);
+
+		wcontext=saved_wcontext;
+		rcontext=saved_rcontext;
+		method_frame=saved_method_frame;
+	}
+
+	template<typename Frame> void call_write(Frame& frame){
+		VMethodFrame *saved_method_frame=method_frame;
+		Value* saved_rcontext=rcontext;
+
+		rcontext=method_frame=&frame;
+
+		frame.call(*this);
+
+		rcontext=saved_rcontext;
+		method_frame=saved_method_frame;
+	}
+
+	Value& construct(VStateless_class &class_value, const Method &method);
 
 	/// execute ops with anti-recoursion check
-	void recoursion_checked_execute(/*const String& name, */ArrayOperation& ops) {
-		// anti_endless_execute_recoursion
-		if(++anti_endless_execute_recoursion==ANTI_ENDLESS_EXECUTE_RECOURSION) {
+	void recoursion_checked_execute(ArrayOperation& ops) {
+		if(++anti_endless_execute_recoursion==pa_execute_recoursion_limit) {
 			anti_endless_execute_recoursion=0; // give @exception a chance
-			throw Exception(PARSER_RUNTIME,
-				0, //&name,
-				"call canceled - endless recursion detected");
+			throw Exception(PARSER_RUNTIME, 0, "call canceled - endless recursion detected");
 		}
 		execute(ops); // execute it
 		anti_endless_execute_recoursion--;
 	}
 
 	///
-	void use_file_directly(VStateless_class& aclass,
-				const String& file_spec,
-				bool fail_on_read_problem=true, 
-				bool fail_on_file_absence=true);
-				
+	void use_file_directly(VStateless_class& aclass, const String& file_spec, bool fail_on_read_problem=true, bool fail_on_file_absence=true);
+
 	/// compiles the file, maybe forcing it's class @a name and @a base_class.
 	void use_file(VStateless_class& aclass, const String& file_name, const String* use_filespec);
-	void use_file(VStateless_class& aclass, const String& file_name, const String* use_filespec, Operation::Origin origin);
+
+	/// for @USE only, calls ^use (which may be user-defined)
+	void use_file(const String& file_name, const String* use_filespec, Operation::Origin origin);
 
 	/// compiles a @a source buffer
-	void use_buf(VStateless_class& aclass,
-		const char* source, 
-		const String* main_alias,
-		uint file_no,
-		int line_no_offset=0);
+	void use_buf(VStateless_class& aclass, const char* source, const String* main_alias, uint file_no, int line_no_offset=0);
 
 	/// processes any code-junction there may be inside of @a value
-	StringOrValue process_getter(Junction& junction); // execute.C
-	StringOrValue process(Value& input_value, bool intercept_string=true); // execute.C
+	Value& process_getter(Junction& junction); // execute.C
+	Value& process(Value& input_value); // execute.C
 	void process_write(Value& input_value); // execute.C
+
 	//@{ convinient helpers
 	const String& process_to_string(Value& input_value) {
-		return process(input_value, true/*intercept_string*/).as_string();
-	}
-	Value& process_to_value(Value& input_value, bool intercept_string=true) {
-		return process(input_value, intercept_string).as_value();
+		return process(input_value).as_string();
 	}
 	//@}
-	const String* get_method_filename(const Method* method); // execute.C
-	const String* get_used_filename(uint file_no);
+	const Operation::Origin get_method_origin(const Method* method); // execute.C
+	const String* get_method_filespec(const Method* method); // execute.C
+	const String* get_used_filespec(uint file_no);
 	
-#define DEFINE_DUAL(modification) \
-	void write_##modification##_lang(StringOrValue dual) { \
-		if(const String* string=dual.get_string()) \
-			write_##modification##_lang(*string); \
-		else \
-			write_##modification##_lang(*dual.get_value()); \
+	/// appending string with it's languages
+	inline void write(const String& astring) {
+		wcontext->write(astring);
+	}
+	
+	/// in [] and {} appending string if get_string is not null, else appending value
+	/// in () appending string if is_string, else appending value
+	inline void write(Value& avalue) {
+		wcontext->write_as_string(avalue);
 	}
 
-	/// appending, sure of clean string inside
-	void write_no_lang(const String& astring) {
-		wcontext->write(astring, 
-			(String::Language)(String::L_CLEAN | (flang & String::L_OPTIMIZE_BIT) ));
+	/// allways appending value
+	inline void write_value(Value& avalue) {
+		wcontext->write(avalue);
 	}
-	/// appending sure value, that would be converted to clean string
-	void write_no_lang(Value& avalue) {
-		if(wcontext->get_in_expression())
-			wcontext->write(avalue);
-		else
-			wcontext->write(avalue, 
-				(String::Language)(String::L_CLEAN | (flang & String::L_OPTIMIZE_BIT) ));
-	}
-
-	/// appending string, passing language built into string being written
-	void write_pass_lang(const String& astring) {
-		wcontext->write(astring, String::L_PASS_APPENDED); 
-	}
-	/// appending possible string, passing language built into string being written
-	void write_pass_lang(Value& avalue) {
-		wcontext->write(avalue, String::L_PASS_APPENDED); 
-	}
-	DEFINE_DUAL(pass)
-
-	/// appending possible string, assigning untaint language
-	void write_assign_lang(Value& avalue) {
-		wcontext->write(avalue, flang); 
-	}
-	/// appending string, assigning untaint language
-	void write_assign_lang(const String& astring) {
-		wcontext->write(astring, flang); 
-	}
-	DEFINE_DUAL(assign)
 
 	/// returns relative to @a path  path to @a file 
 	const String& relative(const char* apath, const String& relative_name);
@@ -299,20 +291,17 @@ public:
 	const String& mime_type_of(const char* user_file_name_cstr);
 
 	/// returns current SQL connection if any
-	SQL_Connection* connection(bool fail_on_error=true) { 
+	SQL_Connection* connection(bool fail_on_error=true) {
 		if(fail_on_error && !fconnection)
-			throw Exception(PARSER_RUNTIME,
-				0,
-				"outside of 'connect' operator");
-
-		return fconnection; 
+			throw Exception(PARSER_RUNTIME, 0, "outside of 'connect' operator");
+		return fconnection;
 	}
 
-	void set_interrupted(bool ainterrupted) { finterrupted=ainterrupted; }
-	bool get_interrupted() { return finterrupted; }
-
-	void set_skip(Skip askip) { fskip=askip; }
 	Skip get_skip() { return fskip; }
+	void set_skip(Skip askip) { fskip=askip; }
+	void set_skip_return(VMethodFrame& amethod_frame) { fskip=SKIP_RETURN; freturn_method_frame=&amethod_frame; }
+	inline bool check_skip_break() { bool result=fskip >= SKIP_BREAK; if(fskip <= SKIP_BREAK) fskip=SKIP_NOTHING; return result; }
+	inline void check_skip_return() { if(fskip==SKIP_RETURN && method_frame==freturn_method_frame) fskip=SKIP_NOTHING; }
 
 	void set_in_cycle(int adelta) { fin_cycle+=adelta; }
 	bool get_in_cycle() { return fin_cycle>0; }
@@ -348,15 +337,8 @@ public: // status read methods
 
 	VMethodFrame *get_method_frame() { return method_frame; }
 	Value& get_self();
-#define GET_SELF(request, type) (static_cast<type &>(request.get_self()))
-	/* for strange reason call to this: 
-		r.get_self<VHash>() 
-		refuses to compile
 
-	template<typename T> T& get_self() {
-		return *static_cast<T*>(get_self().get());
-	}
-	*/
+#define GET_SELF(request, type) (static_cast<type &>(request.get_self()))
 
 	/// public for ^reflection:copy[]
 	void put_element(Value& ncontext, const String& name, Value* value);
@@ -364,22 +346,16 @@ public: // status read methods
 	/// for @main[]
 	const String* execute_virtual_method(Value& aself, const String& method_name);
 
-	/// executes parser method, use op_call(frame) to execute native method
-	void execute_method(VMethodFrame& aframe);
-
 	//{ for @conf[filespec] and @auto[filespec] and parser://method/call
-	const String* execute_method(Value& aself, 
-		const Method& method, Value* optional_param,
-		bool do_return_string);
+	const String* execute_method(Value& aself, const Method& method, Value* optional_param, bool do_return_string);
+
 	struct Execute_nonvirtual_method_result {
 		const String* string;
 		Method* method;
 		Execute_nonvirtual_method_result(): string(0), method(0) {}
 	};
-	Execute_nonvirtual_method_result execute_nonvirtual_method(VStateless_class& aclass, 
-		const String& method_name,
-		VString* optional_param,
-		bool do_return_string);
+
+	Execute_nonvirtual_method_result execute_nonvirtual_method(VStateless_class& aclass, const String& method_name, VString* optional_param, bool do_return_string);
 	//}
 
 #ifdef XML
@@ -405,10 +381,7 @@ private:
 
 private: // compile.C
 
-	ArrayClass& compile(VStateless_class* aclass, 
-		const char* source, const String* main_alias, 
-		uint file_no,
-		int line_no_offset);
+	ArrayClass& compile(VStateless_class* aclass, const char* source, const String* main_alias, uint file_no, int line_no_offset);
 
 private: // execute.C
 
@@ -426,17 +399,6 @@ private: // mime types
 	/// $MAIN:MIME-TYPES
 	Table *mime_types;
 
-private: // lang manipulation
-
-	String::Language set_lang(String::Language alang) {
-		String::Language result=flang;
-		flang=alang;
-		return result;
-	}
-	void restore_lang(String::Language alang) {
-		flang=alang;
-	}
-
 private: // connection manipulation
 
 	SQL_Connection* set_connection(SQL_Connection* aconnection) {
@@ -473,10 +435,10 @@ class Request_context_saver {
 	SQL_Connection* fconnection;
 
 public:
-	Request_context_saver(Request& ar) : 
+	Request_context_saver(Request& ar) :
 		fr(ar),
-		exception_trace_top(ar.exception_trace.top_index()),	
-		exception_trace_bottom(ar.exception_trace.bottom_index()),	
+		exception_trace_top(ar.exception_trace.top_index()),
+		exception_trace_bottom(ar.exception_trace.bottom_index()),
 		stack(ar.stack.top_index()),
 		anti_endless_execute_recoursion(ar.anti_endless_execute_recoursion),
 		method_frame(ar.method_frame),
@@ -495,30 +457,16 @@ public:
 	}
 };
 
-///	Auto-object used for temporary changing Request::flang.
-class Temp_lang {
-	Request& frequest;
-	String::Language saved_lang;
-public:
-	Temp_lang(Request& arequest, String::Language alang) : 
-		frequest(arequest),
-		saved_lang(arequest.set_lang(alang)) {
-	}
-	~Temp_lang() { 
-		frequest.restore_lang(saved_lang); 
-	}
-};
-
 ///	Auto-object used for temporary changing Request::fconnection.
 class Temp_connection {
 	Request& frequest;
 	SQL_Connection* saved_connection;
 public:
-	Temp_connection(Request& arequest, SQL_Connection* aconnection) : 
+	Temp_connection(Request& arequest, SQL_Connection* aconnection) :
 		frequest(arequest),
 		saved_connection(arequest.set_connection(aconnection)) {
 	}
-	~Temp_connection() { 
+	~Temp_connection() {
 		frequest.restore_connection(saved_connection); 
 	}
 };
@@ -530,11 +478,28 @@ public:
 	InCycle(Request& arequest) : frequest(arequest) {
 		frequest.set_in_cycle(1);
 	}
-	~InCycle() { 
+	~InCycle() {
 		frequest.set_in_cycle(-1);
 	}
 };
 
+///	Auto-object used for break out of cycle check
+class TempSkip4Delimiter {
+	Request& frequest;
+	Request::Skip fskip;
+public:
+	TempSkip4Delimiter(Request& arequest) : frequest(arequest), fskip(arequest.get_skip()) {
+		frequest.set_skip(Request::SKIP_NOTHING);
+	}
+	// returns true if break required, should be called
+	bool check_break() {
+		if(frequest.get_skip())
+			fskip=frequest.get_skip();
+		frequest.set_skip(fskip <= Request::SKIP_BREAK ? Request::SKIP_NOTHING : fskip);
+		return fskip >= Request::SKIP_BREAK;
+	}
+};
+
 ///	Auto-object used for temporary changing Request::allow_class_replace.
 class Temp_class_replace {
 	Request& frequest;
