@@ -1,14 +1,14 @@
 /** @file
 	Parser: generally used types & constants decls.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_TYPES_H
 #define PA_TYPES_H
 
-#define IDENT_PA_TYPES_H "$Id: pa_types.h,v 1.52 2015/10/26 01:21:56 moko Exp $"
+#define IDENT_PA_TYPES_H "$Id: pa_types.h,v 1.54 2017/02/07 22:00:37 moko Exp $"
 
 #include "pa_config_includes.h"
 
@@ -33,7 +33,24 @@ typedef unsigned int uint;
 typedef unsigned long ulong;
 //}@
 
-/// max value of integral type
-#define max_integral(type) ((1<<sizeof(type)*8)-1)
+#ifndef LLONG_MAX
+#define LLONG_MAX 9223372036854775807LL
+#endif
+
+#ifndef ULLONG_MAX
+#define ULLONG_MAX 18446744073709551615ULL
+#endif
+
+#ifndef INT32_MAX
+#define INT32_MAX 2147483647L
+#endif
+
+#ifndef UINT32_MAX
+#define UINT32_MAX 4294967295UL
+#endif
+
+#ifndef SSIZE_MAX
+#define SSIZE_MAX (sizeof(size_t)==4 ? INT32_MAX : LLONG_MAX)
+#endif
 
 #endif
