@@ -1,14 +1,14 @@
 /** @file
 	Parser: compiler support helper functions decls.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef COMPILE_TOOLS
 #define COMPILE_TOOLS
 
-#define IDENT_COMPILE_TOOLS_H "$Id: compile_tools.h,v 1.109 2015/10/26 01:21:57 moko Exp $"
+#define IDENT_COMPILE_TOOLS_H "$Id: compile_tools.h,v 1.113 2017/02/07 22:00:40 moko Exp $"
 
 #include "pa_opcode.h"
 #include "pa_types.h"
@@ -18,11 +18,6 @@
 /// used to track source column number
 #define TAB_SIZE 8
 
-#define METHOD_CALL_TYPE_STATIC "static"
-#define METHOD_CALL_TYPE_DYNAMIC "dynamic"
-const String method_call_type_static(METHOD_CALL_TYPE_STATIC);
-const String method_call_type_dynamic(METHOD_CALL_TYPE_DYNAMIC);
-
 enum lexical_state {
 	LS_USER, LS_NAME_SQUARE_PART,
 	LS_USER_COMMENT,
@@ -133,18 +128,17 @@ public:
 			append=false;
 			// append to request's classes
 			if(!request.allow_class_replace)
-				return request.classes().put_dont_replace(cclass->name(), cclass) != 0;
-			request.classes().put(cclass->name(), cclass);
+				return request.classes().put_dont_replace(cclass->type(), cclass) != 0;
+			request.classes().put(cclass->type(), cclass);
 		}
 		return false;
 	}
 
 	VStateless_class* get_existed_class(VStateless_class* aclass){
 		// checking existence of the class during processing @OPTIONS\npartial
-		// method should't use get_class because the last one will call operator @autouse[] if the class wasn't loaded
+		// can't use get_class because it will call @autouse[] if the class wasn't loaded
 		if(aclass)
-			if(Value* class_value=request.classes().get(aclass->name()))
-				return class_value->get_class();
+			return request.classes().get(aclass->type());
 		return 0;
 	}
 
