@@ -1,14 +1,14 @@
 /** @file
 	Parser: Junction class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_JUNCTION_H
 #define PA_JUNCTION_H
 
-#define IDENT_PA_JUNCTION_H "$Id: pa_junction.h,v 1.12 2015/10/26 01:22:00 moko Exp $"
+#define IDENT_PA_JUNCTION_H "$Id: pa_junction.h,v 1.14 2017/02/07 22:00:46 moko Exp $"
 
 
 #include "pa_string.h"
@@ -72,8 +72,8 @@ public:
 		String* aauto_name=0
 	): self(aself),
 		method(amethod),
-		is_getter(ais_getter),
-		auto_name(aauto_name)
+		auto_name(aauto_name),
+		is_getter(ais_getter)
 	 	{}
 
 
