@@ -1,14 +1,14 @@
 /** @file
 	Parser: Method class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_METHOD_H
 #define PA_METHOD_H
 
-#define IDENT_PA_METHOD_H "$Id: pa_method.h,v 1.23 2015/10/26 01:22:00 moko Exp $"
+#define IDENT_PA_METHOD_H "$Id: pa_method.h,v 1.30 2017/02/07 22:00:46 moko Exp $"
 
 #define OPTIMIZE_CALL
 #define OPTIMIZE_RESULT
@@ -59,30 +59,25 @@ public:
 
 	enum Call_optimization {
 		CO_NONE,
-		CO_WITHOUT_FRAME, // for some native methods method frame is not required, faster
-		CO_WITHOUT_WCONTEXT // for some native methods wcontext is not required, faster
+		CO_WITHOUT_FRAME, // for some native methods method frame is not required, faster but $self is unavailable
+		CO_WITHOUT_WCONTEXT // for all native methods wcontext is not required, as no $result, faster
 	};
 
-	///
 	Call_type call_type;
-	//@{
-	/// @name either numbered params // for native-code methods = operators
-	int min_numbered_params_count, max_numbered_params_count;
-	//@}
-	//@{
-	/// @name or named params&locals // for parser-code methods
-	ArrayString* params_names;  ArrayString* locals_names;
-	//@}
-	//@{
-	/// @name the Code
-	ArrayOperation* parser_code;/*OR*/NativeCodePtr native_code;
-	//@}
 
-	VJunction *junction_template;
+	/// either numbered params for native-code methods = operators
+	int min_numbered_params_count, max_numbered_params_count;
+
+	/// or named params&locals for parser-code methods
+	ArrayString* params_names;
+	int params_count;
+	ArrayString* locals_names;
+
+	/// the Code
+	ArrayOperation* parser_code; /*OR*/ NativeCodePtr native_code;
 
 	bool all_vars_local; // in local vars list 'locals' was specified: all vars are local
-	bool allways_use_result; // write to $result detected. will not collect all writes to output scope.
-	String *extra_params; // method has *name as an argument
+
 #ifdef OPTIMIZE_RESULT
 	Result_optimization result_optimization;
 #endif
@@ -91,6 +86,11 @@ public:
 	Call_optimization call_optimization;
 #endif
 
+	mutable VJunction *junction_template;
+
+	const String *name; // method name, never null
+	const String *extra_params; // method has *name as an argument
+
 	Method(
 		Call_type acall_type,
 		int amin_numbered_params_count, int amax_numbered_params_count,
@@ -106,22 +106,24 @@ public:
 		) :
 
 		call_type(acall_type),
-		min_numbered_params_count(amin_numbered_params_count),
-		max_numbered_params_count(amax_numbered_params_count),
-		params_names(aparams_names), locals_names(alocals_names),
+		min_numbered_params_count(amin_numbered_params_count), max_numbered_params_count(amax_numbered_params_count),
+		params_names(aparams_names), params_count(0), locals_names(alocals_names),
 		parser_code(aparser_code), native_code(anative_code),
+		all_vars_local(aall_vars_local),
 #ifdef OPTIMIZE_RESULT
 		result_optimization(aresult_optimization),
 #endif
 #ifdef OPTIMIZE_CALL
-		call_optimization(acall_optimization),  
+		call_optimization(acall_optimization),
 #endif
-		all_vars_local(aall_vars_local){
+		junction_template(0),
+		name(&String::Empty) {
 			if (params_names){
-				const char *last_param = params_names->get(params_names->count()-1)->cstr();
+				params_count=params_names->count();
+				const char *last_param = params_names->get(params_count-1)->cstr();
 				if (last_param[0] == '*' && last_param[1]){
 					extra_params = new String(pa_strdup(last_param+1));
-					params_names->remove(params_names->count()-1);
+					params_names->remove(--params_count);
 					return;
 				}
 			}
@@ -129,10 +131,9 @@ public:
 	}
 
 	/// call this before invoking to ensure proper actual numbered params count
-	void check_actual_numbered_params(
-		Value& self, MethodParams* actual_numbered_params) const;
+	void check_actual_numbered_params(Value& self, MethodParams* actual_numbered_params) const;
 
-	VJunction* get_vjunction(Value& aself) {
+	VJunction* get_vjunction(Value& aself) const {
 		if(!junction_template)
 			return junction_template=new VJunction(aself, this);
 		return junction_template->get(aself);
