@@ -1,14 +1,16 @@
 /**	@file
 	Parser: @b class parser class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VCLASS_H
 #define PA_VCLASS_H
 
-#define IDENT_PA_VCLASS_H "$Id: pa_vclass.h,v 1.64 2015/10/26 01:22:01 moko Exp $"
+#define IDENT_PA_VCLASS_H "$Id: pa_vclass.h,v 1.70 2017/02/07 22:00:47 moko Exp $"
+
+#define OBJECT_PROTOTYPE
 
 // includes
 
@@ -21,7 +23,11 @@
 class VClass: public VStateless_class {
 public: // Value
 	
-	override const char* type() const { return name_cstr(); }
+	override const char* type() const {
+		if(!ftype)
+			throw Exception(PARSER_RUNTIME, 0, "getting type of nameless class");
+		return ftype;
+	}
 
 	/// VClass: true
 	override bool as_bool() const { return true; }
@@ -39,8 +45,12 @@ public: // Value
 	override HashStringValue *get_hash();
 	override HashStringValue* get_fields() { return get_hash(); };
 
-public: 
-	
+public:
+
+#ifdef OBJECT_PROTOTYPE
+	static bool prototype;
+#endif
+
 	// VStateless_class
 	override void real_set_method(const String& aname, Method* amethod);
 	override HashStringProperty* get_properties(){ return &ffields; };
@@ -56,6 +66,9 @@ public:
 
 private:
 
+	const char* ftype;
+	const String& ffilespec;
+
 	enum State {
 		IS_GETTER_ACTIVE = 0x01,
 		IS_SETTER_ACTIVE = 0x02
@@ -69,7 +82,9 @@ private:
 
 public:
 
-	VClass() : state(IS_GETTER_ACTIVE){}
+	VClass(const char* atype, const String* afilespec=0) : ftype(atype), ffilespec(afilespec ? *afilespec : String::Empty), state(IS_GETTER_ACTIVE){}
+
+	const String& get_filespec(){ return ffilespec; }
 
 };
 
