@@ -1,14 +1,14 @@
 /** @file
 	Parser: @b date parser class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VDATE_H
 #define PA_VDATE_H
 
-#define IDENT_PA_VDATE_H "$Id: pa_vdate.h,v 1.60 2015/10/26 01:22:01 moko Exp $"
+#define IDENT_PA_VDATE_H "$Id: pa_vdate.h,v 1.65 2017/02/07 22:00:47 moko Exp $"
 
 #include "classes.h"
 #include "pa_common.h"
@@ -58,6 +58,9 @@ public: // Value
 	/// VDate: method,field
 	override Value* get_element(const String& aname);
 
+	/// VDate: field
+	override const VJunction* put_element(const String& aname, Value* avalue);
+
 public: // usage
 
 	VDate(pa_time_t adate) : ftz_cstr(0) {
@@ -77,19 +80,21 @@ public: // usage
 
 	pa_time_t get_time() const { return ftime; }
 	tm get_tm() const { return ftm; }
+	const char *get_tz() const { return ftz_cstr; }
 
 	enum sql_string_type {sql_string_datetime, sql_string_date, sql_string_time};
+	enum iso_string_type {iso_string_default = 0, iso_string_ms = 1, iso_string_no_colon = 2, iso_string_no_z = 4};
 
 	const String* get_sql_string(sql_string_type format = sql_string_datetime);
+	const String* get_iso_string(iso_string_type format = iso_string_default);
 	const String* get_gmt_string();
-	const String* get_iso_string();
 
 	struct yw {
 		int year;
 		int week;
 	}; 
 	
-	static yw CalcWeek(tm& tms);
+	static yw CalcWeek(tm tms);
 	static int getMonthDays(int year, int month);
 
 private:
