@@ -1,14 +1,14 @@
 /** @file
 	Parser: @b double parser class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VDOUBLE_H
 #define PA_VDOUBLE_H
 
-#define IDENT_PA_VDOUBLE_H "$Id: pa_vdouble.h,v 1.63 2015/11/16 18:27:15 moko Exp $"
+#define IDENT_PA_VDOUBLE_H "$Id: pa_vdouble.h,v 1.67 2017/02/08 13:05:46 moko Exp $"
 
 // includes
 
@@ -22,7 +22,7 @@
 
 // double validation defines
 
-#if _MSC_VER
+#ifdef _MSC_VER
 #include <float.h>
 #define pa_isnan(d) _isnan(d)
 #define pa_finite(d) _finite(d)
@@ -47,8 +47,7 @@ public: // Value
 	/// VDouble: clone
 	override Value& as_expr_result() { return *new VDouble(fdouble); }
 
-	/** VDouble: fdouble 
-	*/
+	/// VDouble: fdouble
 	override const String* get_string() {
 		char local_buf[MAX_NUMBER];
 		size_t length=snprintf(local_buf, MAX_NUMBER, "%.15g", fdouble);
@@ -65,7 +64,7 @@ public: // Value
 
 public: // usage
 
-	VDouble(double adouble): fdouble(adouble) {
+	VDouble(double adouble): fdouble(adouble == 0 ? 0 : adouble) {
 		if(!pa_finite(adouble))
 			throw Exception("number.format", 0, pa_isnan(adouble) ? "invalid number (double)" : "out of range (double)");
 	}
