@@ -1,14 +1,14 @@
 /** @file
 	Parser: @b file parser type decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VFILE_H
 #define PA_VFILE_H
 
-#define IDENT_PA_VFILE_H "$Id: pa_vfile.h,v 1.78 2015/10/26 01:22:01 moko Exp $"
+#define IDENT_PA_VFILE_H "$Id: pa_vfile.h,v 1.84 2017/02/07 22:00:48 moko Exp $"
 
 // include
 
@@ -63,6 +63,9 @@ public: // Value
 	/// VFile: method,field
 	override Value* get_element(const String& aname);
 
+	/// VFile: fields
+	override HashStringValue *get_hash();
+
 public: // usage
 
 	VFile(): fvalue_ptr(0), fvalue_size(0), ftext_tainted(false), fis_text_mode(false), fis_text_content(false){}
@@ -72,7 +75,7 @@ public: // usage
 	/// WARNING: when setting text files be sure to append terminating zero to avalue_ptr
 	/// WARNING: the content can be modified while creating "text" vfile
 	void set(bool atainted, bool ais_text_mode, char* avalue_ptr, size_t avalue_size, const String* afile_name=0, Value* acontent_type=0, Request* r=0);
-	void set(VFile& avfile, bool aset_text_mode, bool ais_text_mode, const String* afile_name=0, Value* acontent_type=0, Request* r=0);
+	void set(VFile& avfile, bool *ais_text_mode, const String* afile_name=0, Value* acontent_type=0, Request* r=0);
 	void set_binary(bool atainted, const char* avalue_ptr, size_t avalue_size, const String* afile_name=0, Value* acontent_type=0, Request* r=0);
 	void set_binary_string(bool atainted, const char* avalue_ptr, size_t avalue_size);
 
@@ -88,9 +91,13 @@ public: // usage
 	}
 
 	size_t value_size() const { return fvalue_size; }
+	bool is_text_mode() const { return fis_text_mode; }
 
 	HashStringValue& fields() { return ffields; }
 
+	Charset* detect_binary_charset(Charset* charset);
+	void transcode(Charset& from_charset, Charset& to_charset);
+
 private:
 	const char* text_cstr();
 
