@@ -1,14 +1,14 @@
 /**	@file
 	Parser: @b object class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VOBJECT_H
 #define PA_VOBJECT_H
 
-#define IDENT_PA_VOBJECT_H "$Id: pa_vobject.h,v 1.64 2015/10/26 01:22:02 moko Exp $"
+#define IDENT_PA_VOBJECT_H "$Id: pa_vobject.h,v 1.67 2017/02/07 22:00:50 moko Exp $"
 
 // includes
 
@@ -39,7 +39,7 @@ class VObject: public Value {
 
 public: // Value
 	
-	const char* type() const { return fclass.name_cstr(); }
+	const char* type() const { return fclass.type(); }
 	override Value* as(const char* atype);
 
 	/// VObject: fclass
@@ -55,6 +55,7 @@ public: // Value
 	override HashStringValue* get_hash();
 	override Table *get_table();
 	override HashStringValue* get_fields() { return &ffields; }
+	override HashStringValue* get_fields_reference() { return &ffields; }
 
 	override Value* get_element(const String& aname);
 	override const VJunction* put_element(const String& name, Value* value);
