@@ -1,14 +1,14 @@
 /** @file
 	Parser: stateless class decls.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VSTATELESS_CLASS_H
 #define PA_VSTATELESS_CLASS_H
 
-#define IDENT_PA_VSTATELESS_CLASS_H "$Id: pa_vstateless_class.h,v 1.78 2015/10/26 01:22:02 moko Exp $"
+#define IDENT_PA_VSTATELESS_CLASS_H "$Id: pa_vstateless_class.h,v 1.91 2017/02/15 17:21:28 moko Exp $"
 
 // include
 
@@ -17,20 +17,12 @@
 #include "pa_vjunction.h"
 #include "pa_method.h"
 
-// defines
-
-#define CLASS_NAME "CLASS"
-#ifndef OPTIMIZE_BYTECODE_GET_ELEMENT__SPECIAL
-#define CLASS_NAMETEXT "CLASS_NAME"
-extern const String class_name, class_nametext;
-#endif
-
 // forwards
 
 class VStateless_class;
 typedef Array<VStateless_class*> ArrayClass;
-typedef HashString<Property *> HashStringProperty;
-typedef HashString<Method *> HashStringMethod;
+typedef HASH_STRING<Property *> HashStringProperty;
+typedef HASH_STRING<Method *> HashStringMethod;
 
 class Temp_method;
 
@@ -44,8 +36,6 @@ class Temp_method;
 class VStateless_class: public Value {
 	friend class Temp_method;
 
-	const String* fname;
-	mutable const char* fname_cstr;
 	HashStringMethod fmethods;
 
 	bool flocked;
@@ -65,8 +55,6 @@ protected:
 
 public: // Value
 	
-	const char* type() const { return "stateless_class"; }
-
 	/// VStateless_class: this
 	override VStateless_class* get_class() { return this; }
 	/// VStateless_class: fbase
@@ -76,9 +64,9 @@ public: // Value
 	/// get_element with aself for VObject junctions
 	virtual Value* get_element(Value& aself, const String& aname);
 
-	override const VJunction* put_element(const String& aname, Value* avalue) {	return put_element(*this, aname, avalue); }
+	override const VJunction* put_element(const String& aname, Value* avalue) { return put_element(*this, aname, avalue); }
 	/// put_element with aself for VObject junctions
-	virtual const VJunction* put_element(Value& aself, const String& aname, Value* /*avalue*/) {	
+	virtual const VJunction* put_element(Value& aself, const String& aname, Value* /*avalue*/) {
 		aself.bark("element can not be stored to %s", &aname); 
 		return 0;
 	}
@@ -100,53 +88,28 @@ public: // Value
 
 public: // usage
 
-	VStateless_class(
-		const String* aname=0, 
-		VStateless_class* abase=0):
-		fname(aname),
+	VStateless_class(VStateless_class* amethoded_donor=0):
 		flocked(false),
-		fbase(0),
-		fderived(0),
 		fall_vars_local(false),
 		fpartial(false),
+		fcall_type(Method::CT_ANY),
+		fbase(0),
+		fderived(0),
 		fscalar(0),
 		fdefault_getter(0),
-		fdefault_setter(0),
-		fcall_type(Method::CT_ANY)
-		{
-			set_base(abase);
+		fdefault_setter(0)
+	{
+		if(amethoded_donor)
+			fmethods.merge_dont_replace(amethoded_donor->fmethods);
 	}
 
 	void lock() { flocked=true; }
 
-	const String& name() const { 
-		if(!fname) {
-			if(fbase)
-				return fbase->name();
-
-			throw Exception(PARSER_RUNTIME,
-				0,
-				"getting name of nameless class");
-		}
-		return *fname; 
-	}
-
-	const char* name_cstr() const{
-		if(!fname_cstr) // remembering last calculated, and can't reassign 'fname_cstr'!
-			fname_cstr=name().cstr();
-		return fname_cstr;
-	}
-
-	void set_name(const String& aname) {
-		fname=&aname; 
-		fname_cstr=0;
-	}
-
-	Method* get_method(const String& aname) const { 
+	Method* get_method(const String::Body &aname) const {
 		return fmethods.get(aname);
 	}
 
-	HashStringMethod get_methods(){
+	HashStringMethod& get_methods(){
 		return fmethods;
 	}
 
@@ -172,10 +135,7 @@ public: // usage
 
 	void set_methods_call_type(Method::Call_type call_type){
 		if(fcall_type!=Method::CT_ANY)
-			throw Exception(PARSER_RUNTIME,
-						0,
-						"You can specify call type option in a class only once"
-					);
+			throw Exception(PARSER_RUNTIME, 0, "You can specify call type option in a class only once");
 		fcall_type=call_type;
 	}
 
@@ -183,8 +143,8 @@ public: // usage
 		const char* cstr_name,
 		Method::Call_type call_type,
 		NativeCodePtr native_code,
-		int min_numbered_params_count, 
-		int max_numbered_params_count, 
+		int min_numbered_params_count,
+		int max_numbered_params_count,
 		Method::Call_optimization call_optimization=Method::CO_WITHOUT_WCONTEXT);
 
 	void set_method(const String& aname, Method* amethod);
@@ -210,7 +170,7 @@ class Temp_method {
 	const String& fname;
 	Method* saved_method;
 public:
-	Temp_method(VStateless_class& aclass, const String& aname, Method* amethod) : 
+	Temp_method(VStateless_class& aclass, const String& aname, Method* amethod) :
 		fclass(aclass),
 		fname(aname),
 		saved_method(aclass.get_method(aname)) {
@@ -221,4 +181,25 @@ public:
 	}
 };
 
+
+class VBaseClassWrapper: public Value {
+
+	VStateless_class& fclass;
+	Value& fself;
+
+public: // Value
+	
+	override const char* type() const { return fclass.type(); }
+	override VStateless_class* get_class() { return &fclass; }
+	override VStateless_class* base() { return fclass.base(); }
+
+	override Value* get_element(const String& aname) { return fclass.get_element(fself, aname); }
+	override const VJunction* put_element(const String& aname, Value* avalue) { return fclass.put_element(fself, aname, avalue); }
+	override Value& as_expr_result(){ return fclass.as_expr_result(); }
+
+public: // usage
+
+	VBaseClassWrapper(VStateless_class& aclass, Value& aself): fclass(aclass), fself(aself) {};
+};
+
 #endif
