@@ -1,30 +1,25 @@
 /** @file
 	Parser: @b status class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VSTATUS_H
 #define PA_VSTATUS_H
 
-#define IDENT_PA_VSTATUS_H "$Id: pa_vstatus.h,v 1.22 2015/10/26 01:22:03 moko Exp $"
+#define IDENT_PA_VSTATUS_H "$Id: pa_vstatus.h,v 1.24 2017/02/07 22:00:51 moko Exp $"
 
 // includes
 
-#include "pa_value.h"
-
-// define
-
-#define STATUS_CLASS_NAME "status"
+#include "pa_vstateless_class.h"
 
 /// status class
-class VStatus: public Value {
+class VStatus: public VStateless_class {
 
 public: // Value
 	
 	const char* type() const { return "status"; }
-	VStateless_class *get_class() { return 0; }
 
 	// VStatus: field
 	Value* get_element(const String& aname);
