@@ -1,14 +1,14 @@
 /** @file
 	Parser: @b void parser class.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VVOID_H
 #define PA_VVOID_H
 
-#define IDENT_PA_VVOID_H "$Id: pa_vvoid.h,v 1.42 2015/10/26 01:22:03 moko Exp $"
+#define IDENT_PA_VVOID_H "$Id: pa_vvoid.h,v 1.44 2017/02/07 22:00:52 moko Exp $"
 
 #define STRICT_VARS
 
@@ -40,14 +40,6 @@ public: // Value
 		}
 	}
 
-	/// VVoid: methods
-	override Value* get_element(const String& aname) {
-		// methods
-		if(Value* result=VStateless_object::get_element(aname))
-			return result;
-		return 0;
-	}
-
 #ifdef STRICT_VARS
 	static bool strict_vars;
 #define CHECK_STRICT if(strict_vars) throw Exception(PARSER_RUNTIME, 0, "Use of uninitialized value");
