@@ -1,14 +1,14 @@
 /** @file
 	Parser: @b xdoc parser class decl.
 
-	Copyright (c) 2001-2015 Art. Lebedev Studio (http://www.artlebedev.com)
+	Copyright (c) 2001-2017 Art. Lebedev Studio (http://www.artlebedev.com)
 	Author: Alexandr Petrosian <paf@design.ru> (http://paf.design.ru)
 */
 
 #ifndef PA_VXDOC_H
 #define PA_VXDOC_H
 
-#define IDENT_PA_VXDOC_H "$Id: pa_vxdoc.h,v 1.54 2015/10/26 01:22:03 moko Exp $"
+#define IDENT_PA_VXDOC_H "$Id: pa_vxdoc.h,v 1.58 2017/02/07 22:00:52 moko Exp $"
 
 #include "classes.h"
 #include "pa_common.h"
@@ -42,6 +42,7 @@ struct XDocOutputOptions {
 	void append(Request& r, HashStringValue* options, bool with_filename=false);
 };
 
+typedef struct _xsltStylesheet xsltStylesheet;
 
 /// value of type 'xdoc'. implemented with libxml & co
 class VXdoc: public VXnode {
@@ -66,7 +67,7 @@ public: // Value
 
 public: // VXNode
 
-	override xmlNode& get_xmlnode() { 
+	override xmlNode& get_xmlnode() {
 		return *reinterpret_cast<xmlNode*>(&get_xmldoc());
 	}
 
@@ -76,10 +77,7 @@ public: // VXNode
 
 public: // usage
 
-	VXdoc() : 
-		VXnode(*this), 
-		fcharsets(0),
-		fdocument(0) {}
+	VXdoc() : VXnode(*this), stylesheet(0), fcharsets(0), fdocument(0) {}
 
 	VXdoc(Request_charsets& acharsets, xmlDoc& adocument) : VXnode(*this) {
 		set_xmldoc(acharsets, adocument);
@@ -91,21 +89,19 @@ public: // VXdoc
 		fcharsets=&acharsets;
 		fdocument=&adocument;
 		fdocument->_private=this;
-	}
-	xmlDoc& get_xmldoc() { 
-		if(!fdocument)
-			throw Exception(PARSER_RUNTIME,
-				0,
-				"using unitialized xdoc object");
-		return *fdocument; 
+		stylesheet=0;
 	}
 
-	Request_charsets& charsets() { 
+	xmlDoc& get_xmldoc() {
+		if(!fdocument)
+			throw Exception(PARSER_RUNTIME, 0, "using unitialized xdoc object");
+		return *fdocument;
+	}
+
+	Request_charsets& charsets() {
 		if(!fcharsets)
-			throw Exception(PARSER_RUNTIME,
-				0,
-				"using unitialized xdoc object");
-		return *fcharsets; 
+			throw Exception(PARSER_RUNTIME, 0, "using unitialized xdoc object");
+		return *fcharsets;
 	}
 
 	VXnode& wrap(xmlNode& anode);
@@ -116,6 +112,8 @@ public:
 
 	XDocOutputOptions output_options;
 
+	xsltStylesheet *stylesheet;
+
 private:
 
 	Request_charsets* fcharsets;
